@@ -200,4 +200,24 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " W a t c h   E x p r e s s i o n s " ]   =   " W a t c h   E x p r e s s i o n s " ; 
  l o c a l i z e d S t r i n g s [ " A d d " ]   =   " A d d " ; 
  l o c a l i z e d S t r i n g s [ " D e l e t e   w a t c h   e x p r e s s i o n . " ]   =   " D e l e t e   w a t c h   e x p r e s s i o n . " ; 
+ l o c a l i z e d S t r i n g s [ " H T T P   I n f o r m a t i o n " ]   =   " H T T P   I n f o r m a t i o n " ; 
+ l o c a l i z e d S t r i n g s [ " R e q u e s t   M e t h o d " ]   =   " R e q u e s t   M e t h o d " ; 
+ l o c a l i z e d S t r i n g s [ " S t a t u s   C o d e " ]   =   " S t a t u s   C o d e " ; 
+ l o c a l i z e d S t r i n g s [ " ( u n a b l e   t o   d e c o d e   v a l u e ) " ]   =   " ( u n a b l e   t o   d e c o d e   v a l u e ) " ; 
+ l o c a l i z e d S t r i n g s [ " E v e n t   L i s t e n e r s " ]   =   " E v e n t   L i s t e n e r s " ; 
+ l o c a l i z e d S t r i n g s [ " N o   E v e n t   L i s t e n e r s " ]   =   " N o   E v e n t   L i s t e n e r s " ; 
+ l o c a l i z e d S t r i n g s [ " S e l e c t e d   N o d e   O n l y " ]   =   " S e l e c t e d   N o d e   O n l y " ; 
+ l o c a l i z e d S t r i n g s [ " A l l   N o d e s " ]   =   " A l l   N o d e s " ; 
+ l o c a l i z e d S t r i n g s [ " L o a d   e v e n t   f i r e d " ]   =   " L o a d   e v e n t   f i r e d " ; 
+ l o c a l i z e d S t r i n g s [ " D O M C o n t e n t   e v e n t   f i r e d " ]   =   " D O M C o n t e n t   e v e n t   f i r e d " ; 
+ 
+ l o c a l i z e d S t r i n g s [ " L o c a l   F i l e s " ]   =   " L o c a l   F i l e s " ; 
+ l o c a l i z e d S t r i n g s [ " T i m e l i n e " ]   =   " T i m e l i n e " ; 
+ l o c a l i z e d S t r i n g s [ " R E C O R D S " ]   =   " R E C O R D S " ; 
+ l o c a l i z e d S t r i n g s [ " D O M   E v e n t " ]   =   " D O M   E v e n t " ; 
+ l o c a l i z e d S t r i n g s [ " L a y o u t " ]   =   " L a y o u t " ; 
+ l o c a l i z e d S t r i n g s [ " R e c a l c u l a t e   S t y l e " ]   =   " R e c a l c u l a t e   S t y l e " ; 
+ l o c a l i z e d S t r i n g s [ " P a i n t " ]   =   " P a i n t " ; 
+ l o c a l i z e d S t r i n g s [ " L a y o u t " ]   =   " L a y o u t " ; 
+ l o c a l i z e d S t r i n g s [ " P a r s e " ]   =   " P a r s e " ; 
  
\ No newline at end of file
