@@ -32,6 +32,19 @@ bool AccessibilityObject::accessibilityIgnoreAttachment() const
     return false;
 }
 
+bool AccessibilityObject::accessibilityPlatformIncludesObject() const
+{
+    // When a list item is made up entirely of children (e.g. paragraphs)
+    // the list item gets ignored. We need it.
+    if (isGroup()) {
+        AccessibilityObject* parent = parentObject();
+        if (parent && parent->isList())
+            return true;
+    }
+
+    return false;
+}
+
 AccessibilityObjectWrapper* AccessibilityObject::wrapper() const
 {
     return m_wrapper;
