@@ -36,7 +36,6 @@
 
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkversion.h>
 
 namespace WebCore {
 
@@ -137,6 +136,8 @@ static String keyIdentifierForGdkKeyCode(guint keyCode)
             // Standard says that DEL becomes U+007F.
         case GDK_Delete:
             return "U+007F";
+        case GDK_BackSpace:
+            return "U+0008";
         case GDK_ISO_Left_Tab:
         case GDK_3270_BackTab:
         case GDK_Tab:
@@ -199,6 +200,7 @@ static int windowsKeyCodeForKeyEvent(unsigned int keycode)
         case GDK_Control_R:
             return VK_CONTROL; // (11) CTRL key
         case GDK_Menu:
+            return VK_APPS;  // (5D) Applications key (Natural keyboard)
         case GDK_Alt_L:
         case GDK_Alt_R:
             return VK_MENU; // (12) ALT key
@@ -258,9 +260,10 @@ static int windowsKeyCodeForKeyEvent(unsigned int keycode)
         case GDK_Help:
             return VK_HELP; // (2F) HELP key
         case GDK_0:
-        case GDK_parenleft:
+        case GDK_parenright:
             return VK_0;    //  (30) 0) key
         case GDK_1:
+        case GDK_exclam:
             return VK_1; //  (31) 1 ! key
         case GDK_2:
         case GDK_at:
@@ -284,7 +287,7 @@ static int windowsKeyCodeForKeyEvent(unsigned int keycode)
         case GDK_asterisk:
             return VK_8; //  (38) 8 key  '*'
         case GDK_9:
-        case GDK_parenright:
+        case GDK_parenleft:
             return VK_9; //  (39) 9 key '('
         case GDK_a:
         case GDK_A:
@@ -368,7 +371,6 @@ static int windowsKeyCodeForKeyEvent(unsigned int keycode)
             return VK_LWIN; // (5B) Left Windows key (Microsoft Natural keyboard)
         case GDK_Meta_R:
             return VK_RWIN; // (5C) Right Windows key (Natural keyboard)
-            // VK_APPS (5D) Applications key (Natural keyboard)
             // VK_SLEEP (5F) Computer Sleep key
             // VK_SEPARATOR (6C) Separator key
             // VK_SUBTRACT (6D) Subtract key
@@ -464,6 +466,32 @@ static int windowsKeyCodeForKeyEvent(unsigned int keycode)
             // VK_NONAME (FC) Reserved for future use
             // VK_PA1 (FD) PA1 key
             // VK_OEM_CLEAR (FE) Clear key
+        case GDK_F1:
+        case GDK_F2:
+        case GDK_F3:
+        case GDK_F4:
+        case GDK_F5:
+        case GDK_F6:
+        case GDK_F7:
+        case GDK_F8:
+        case GDK_F9:
+        case GDK_F10:
+        case GDK_F11:
+        case GDK_F12:
+        case GDK_F13:
+        case GDK_F14:
+        case GDK_F15:
+        case GDK_F16:
+        case GDK_F17:
+        case GDK_F18:
+        case GDK_F19:
+        case GDK_F20:
+        case GDK_F21:
+        case GDK_F22:
+        case GDK_F23:
+        case GDK_F24:
+            return VK_F1 + (keycode - GDK_F1);
+
         default:
             return 0;
     }
@@ -477,6 +505,8 @@ static String singleCharacterString(guint val)
         case GDK_KP_Enter:
         case GDK_Return:
             return String("\r");
+        case GDK_BackSpace:
+            return String("\x8");
         default:
             gunichar c = gdk_keyval_to_unicode(val);
             glong nwc;
@@ -503,16 +533,12 @@ PlatformKeyboardEvent::PlatformKeyboardEvent(GdkEventKey* event)
     , m_keyIdentifier(keyIdentifierForGdkKeyCode(event->keyval))
     , m_autoRepeat(false)
     , m_windowsVirtualKeyCode(windowsKeyCodeForKeyEvent(event->keyval))
+    , m_nativeVirtualKeyCode(event->keyval)
     , m_isKeypad(event->keyval >= GDK_KP_Space && event->keyval <= GDK_KP_9)
     , m_shiftKey((event->state & GDK_SHIFT_MASK) || (event->keyval == GDK_3270_BackTab))
     , m_ctrlKey(event->state & GDK_CONTROL_MASK)
     , m_altKey(event->state & GDK_MOD1_MASK)
-#if GTK_CHECK_VERSION(2,10,0)
     , m_metaKey(event->state & GDK_META_MASK)
-#else
-    // GDK_MOD2_MASK doesn't always mean meta so we can't use it
-    , m_metaKey(false)
-#endif
     , m_gdkEventKey(event)
 {
 }
