@@ -20,7 +20,7 @@
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
  * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #ifndef Storage_h
@@ -44,8 +44,15 @@ namespace WebCore {
         static PassRefPtr<Storage> create(Frame*, PassRefPtr<StorageArea>);
         ~Storage();
 
+#if USE(V8)
+        static void setLocalStorageAvailable(bool);
+        static bool localStorageAvailable();
+        static void setSessionStorageAvailable(bool);
+        static bool sessionStorageAvailable();
+#endif
+
         unsigned length() const;
-        String key(unsigned index, ExceptionCode&) const;
+        String key(unsigned index) const;
         String getItem(const String&) const;
         void setItem(const String& key, const String& value, ExceptionCode&);
         void removeItem(const String&);
@@ -53,11 +60,12 @@ namespace WebCore {
 
         bool contains(const String& key) const;
 
+        Frame* frame() { return m_frame; }
         void disconnectFrame() { m_frame = 0; }
 
     private:
         Storage(Frame*, PassRefPtr<StorageArea>);
-            
+
         Frame* m_frame;
         RefPtr<StorageArea> m_storageArea;
     };
