@@ -18,8 +18,8 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef WEBKIT_WEB_FRAME_H
-#define WEBKIT_WEB_FRAME_H
+#ifndef webkitwebframe_h
+#define webkitwebframe_h
 
 #include <glib-object.h>
 #include <gtk/gtk.h>
@@ -75,12 +75,17 @@ struct _WebKitWebFrameClass {
  * because the data available at the time was not significant enough.
  * @WEBKIT_LOAD_FINISHED: This state means that everything that was
  * required to display the page has been loaded.
+ * @WEBKIT_LOAD_FAILED: This state means that some error occurred
+ * during the page load that prevented it from being completed. You
+ * can connect to the #WebKitWebView::load-error signal if you want to
+ * know precisely what kind of error occurred.
  */
 typedef enum {
     WEBKIT_LOAD_PROVISIONAL,
     WEBKIT_LOAD_COMMITTED,
     WEBKIT_LOAD_FINISHED,
-    WEBKIT_LOAD_FIRST_VISUALLY_NON_EMPTY_LAYOUT
+    WEBKIT_LOAD_FIRST_VISUALLY_NON_EMPTY_LAYOUT,
+    WEBKIT_LOAD_FAILED
 } WebKitLoadStatus;
 
 WEBKIT_API GType
