@@ -53,6 +53,7 @@ extern "C" {
 }
 
 static bool down = false;
+static bool currentEventButton = 1;
 static bool dragMode = true;
 static bool replayingSavedEvents = false;
 static int lastMousePositionX;
@@ -122,12 +123,27 @@ static JSValueRef contextClickCallback(JSContextRef context, JSObjectRef functio
 static void updateClickCount(int /* button */)
 {
     // FIXME: take the last clicked button number and the time of last click into account.
-    if (lastClickPositionX != lastMousePositionX && lastClickPositionY != lastMousePositionY)
+    if (lastClickPositionX != lastMousePositionX || lastClickPositionY != lastMousePositionY)
         clickCount = 1;
     else
         clickCount++;
 }
 
+#if !GTK_CHECK_VERSION(2,17,3)
+static void getRootCoords(GtkWidget* view, int* rootX, int* rootY)
+{
+    GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(view));
+    int tmpX, tmpY;
+
+    gtk_widget_translate_coordinates(view, window, lastMousePositionX, lastMousePositionY, &tmpX, &tmpY);
+
+    gdk_window_get_origin(window->window, rootX, rootY);
+
+    *rootX += tmpX;
+    *rootY += tmpY;
+}
+#endif
+
 static JSValueRef mouseDownCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
 {
     WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
@@ -140,9 +156,29 @@ static JSValueRef mouseDownCallback(JSContextRef context, JSObjectRef function,
     memset(&event, 0, sizeof(event));
     event.type = GDK_BUTTON_PRESS;
     event.button.button = 1;
+
+    if (argumentCount == 1) {
+        event.button.button = (int)JSValueToNumber(context, arguments[0], exception) + 1;
+        g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
+    }
+
+    currentEventButton = event.button.button;
+
     event.button.x = lastMousePositionX;
     event.button.y = lastMousePositionY;
     event.button.window = GTK_WIDGET(view)->window;
+    event.button.time = GDK_CURRENT_TIME;
+    event.button.device = gdk_device_get_core_pointer();
+
+    int x_root, y_root;
+#if GTK_CHECK_VERSION(2,17,3)
+    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &x_root, &y_root);
+#else
+    getRootCoords(GTK_WIDGET(view), &x_root, &y_root);
+#endif
+
+    event.button.x_root = x_root;
+    event.button.y_root = y_root;
 
     updateClickCount(1);
 
@@ -177,9 +213,29 @@ static JSValueRef mouseUpCallback(JSContextRef context, JSObjectRef function, JS
     memset(&event, 0, sizeof(event));
     event.type = GDK_BUTTON_RELEASE;
     event.button.button = 1;
+
+    if (argumentCount == 1) {
+        event.button.button = (int)JSValueToNumber(context, arguments[0], exception) + 1;
+        g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
+    }
+
+    currentEventButton = event.button.button;
+
     event.button.x = lastMousePositionX;
     event.button.y = lastMousePositionY;
     event.button.window = GTK_WIDGET(view)->window;
+    event.button.time = GDK_CURRENT_TIME;
+    event.button.device = gdk_device_get_core_pointer();
+
+    int x_root, y_root;
+#if GTK_CHECK_VERSION(2,17,3)
+    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &x_root, &y_root);
+#else
+    getRootCoords(GTK_WIDGET(view), &x_root, &y_root);
+#endif
+
+    event.button.x_root = x_root;
+    event.button.y_root = y_root;
 
     if ((dragMode && !replayingSavedEvents) || msgQueue[endOfQueue].delay) {
         msgQueue[endOfQueue].event = event;
@@ -213,11 +269,33 @@ static JSValueRef mouseMoveToCallback(JSContextRef context, JSObjectRef function
     g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
 
     GdkEvent event;
+    memset(&event, 0, sizeof(event));
     event.type = GDK_MOTION_NOTIFY;
     event.motion.x = lastMousePositionX;
     event.motion.y = lastMousePositionY;
     event.motion.time = GDK_CURRENT_TIME;
     event.motion.window = GTK_WIDGET(view)->window;
+    event.motion.device = gdk_device_get_core_pointer();
+
+    int x_root, y_root;
+#if GTK_CHECK_VERSION(2,17,3)
+    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &x_root, &y_root);
+#else
+    getRootCoords(GTK_WIDGET(view), &x_root, &y_root);
+#endif
+
+    event.motion.x_root = x_root;
+    event.motion.y_root = y_root;
+
+    if (down) {
+        if (currentEventButton == 1)
+            event.motion.state = GDK_BUTTON1_MASK;
+        else if (currentEventButton == 2)
+            event.motion.state = GDK_BUTTON2_MASK;
+        else if (currentEventButton == 3)
+            event.motion.state = GDK_BUTTON3_MASK;
+    } else
+        event.motion.state = 0;
 
     if (dragMode && down && !replayingSavedEvents) {
         msgQueue[endOfQueue].event = event;
@@ -232,6 +310,52 @@ static JSValueRef mouseMoveToCallback(JSContextRef context, JSObjectRef function
     return JSValueMakeUndefined(context);
 }
 
+static JSValueRef mouseWheelToCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
+{
+    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
+    if (!view)
+        return JSValueMakeUndefined(context);
+
+    if (argumentCount < 2)
+        return JSValueMakeUndefined(context);
+
+    int horizontal = (int)JSValueToNumber(context, arguments[0], exception);
+    g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
+    int vertical = (int)JSValueToNumber(context, arguments[1], exception);
+    g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
+
+    // GTK+ doesn't support multiple direction scrolls in the same event!
+    g_return_val_if_fail((!vertical || !horizontal), JSValueMakeUndefined(context));
+
+    GdkEvent event;
+    event.type = GDK_SCROLL;
+    event.scroll.x = lastMousePositionX;
+    event.scroll.y = lastMousePositionY;
+    event.scroll.time = GDK_CURRENT_TIME;
+    event.scroll.window = GTK_WIDGET(view)->window;
+
+    if (horizontal < 0)
+        event.scroll.direction = GDK_SCROLL_LEFT;
+    else if (horizontal > 0)
+        event.scroll.direction = GDK_SCROLL_RIGHT;
+    else if (vertical < 0)
+        event.scroll.direction = GDK_SCROLL_UP;
+    else if (vertical > 0)
+        event.scroll.direction = GDK_SCROLL_DOWN;
+    else
+        g_assert_not_reached();
+
+    if (dragMode && down && !replayingSavedEvents) {
+        msgQueue[endOfQueue].event = event;
+        msgQueue[endOfQueue++].isDragEvent = true;
+    } else {
+        webkit_web_frame_layout(mainFrame);
+        gtk_main_do_event(&event);
+    }
+
+    return JSValueMakeUndefined(context);
+}
+
 static JSValueRef beginDragWithFilesCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
 {
     if (argumentCount < 1)
@@ -335,7 +459,7 @@ static JSValueRef keyDownCallback(JSContextRef context, JSObjectRef function, JS
     else if (JSStringIsEqualToUTF8CString(character, "end"))
         gdkKeySym = GDK_End;
     else if (JSStringIsEqualToUTF8CString(character, "delete"))
-        gdkKeySym = GDK_Delete;
+        gdkKeySym = GDK_BackSpace;
     else if (JSStringIsEqualToUTF8CString(character, "F1"))
         gdkKeySym = GDK_F1;
     else if (JSStringIsEqualToUTF8CString(character, "F2"))
@@ -443,6 +567,7 @@ static JSValueRef zoomPageOutCallback(JSContextRef context, JSObjectRef function
 }
 
 static JSStaticFunction staticFunctions[] = {
+    { "mouseWheelTo", mouseWheelToCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
     { "contextClick", contextClickCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
     { "mouseDown", mouseDownCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
     { "mouseUp", mouseUpCallback, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete },
